-----------------------------------------------------------------------------------------------
-- Client Lua Script for GuideLine
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "QuestLib"
require "GameLib"
require "Vector3"
 
-----------------------------------------------------------------------------------------------
-- GuideLine Module Definition
-----------------------------------------------------------------------------------------------
local GuideLine = {}

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function GuideLine:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 
	
    -- initialize variables here
	self.tUnits = {} -- ID, type
	self.tTracked = {giver = {}, objective = {}, path = {}, harvest = {}, farm = {}, none = {}}
	
	self.tUnitsDrawn = {}
	
	self.markerSets = {}
	self.unusedMarkerSets = {}
	self.MarkerNames = {}
	self.MarkerNames[1]="Dots"
	self.MarkerNames[2]="Arrows"
	self.MarkerNames[3]="Flag"
		
	--self.Marker = "Dots"
	
	self.lastPosition = nil
	self.wndObjectivesList = {}
	self.quest = nil
	self.shouldUpdatePossibleLines = false
	self.curPosition = nil
	self.player = nil
	self.playerVector = nil
	self.shouldUpdateUnits = false
	self.lineUsedCount = 0
	self.inCombat = false
	self.pvp = false
	self.questCount = 0
	self.tQuestList = {}
	self.tQuestsActive = {}
	self.numPoints = 0
	self.questCarbine = nil
	self.lastObjectiveNum = nil
	self.bIsMiner, self.bIsRelicHunter, self.bIsSurvivalist, self.bIsFarmer = false, false, false, false
	self.pathName = ""
	self.preloadUnits = {}
	self.unitsToCheck = {}
	self.time = nil
	self.marker = {}
	
	
	self.miniMapMarker =
		{
		IronNode				= "harvest",
		TitaniumNode			= "harvest",
		ZephyriteNode			= "harvest",
		PlatinumNode			= "harvest",
		HydrogemNode			= "harvest",
		XenociteNode			= "harvest",
		ShadeslateNode			= "harvest",
		GalactiumNode			= "harvest",
		NovaciteNode			= "harvest",
		StandardRelicNode		= "harvest",
		AcceleratedRelicNode	= "harvest",
		AdvancedRelicNode		= "harvest",
		DynamicRelicNode		= "harvest",
		KineticRelicNode		= "harvest",
		SpirovineNode			= "farm",
		BladeleafNode			= "farm",
		YellowbellNode			= "farm",
		PummelgranateNode		= "farm",
		SerpentlilyNode			= "farm",
		GoldleafNode			= "farm",
		HoneywheatNode			= "farm",
		CrowncornNode			= "farm",
		CoralscaleNode			= "farm",
		LogicleafNode			= "farm",
		StoutrootNode			= "farm",
		GlowmelonNode			= "farm",
		FaerybloomNode			= "farm",
		WitherwoodNode			= "farm",
		FlamefrondNode			= "farm",
		GrimgourdNode			= "farm",
		MourningstarNode		= "farm",
		BloodbriarNode			= "farm",
		OctopodNode				= "farm",
		HeartichokeNode			= "farm",
		SmlGrowthshroomNode		= "farm",
		MedGrowthshroomNode		= "farm",
		LrgGrowthshroomNode		= "farm",
		SmlHarvestshroomNode	= "farm",
		MedHarvestshroomNode	= "farm",
		LrgHarvestshroomNode	= "farm",
		SmlRenewshroomNode		= "farm",
		MedRenewshroomNode		= "farm",
		LrgRenewshroomNode		= "farm",
		AlgorocTreeNode			= "harvest",
		CelestionTreeNode		= "harvest",
		DeraduneTreeNode		= "harvest",
		EllevarTreeNode			= "harvest",
		GalerasTreeNode			= "harvest",
		AuroriaTreeNode			= "harvest",
		WhitevaleTreeNode		= "harvest",
		DreadmoorTreeNode		= "harvest",
		FarsideTreeNode			= "harvest",
		CoralusTreeNode			= "harvest",
		MurkmireTreeNode		= "harvest",
		WilderrunTreeNode		= "harvest",
		MalgraveTreeNode		= "harvest",
		HalonRingTreeNode		= "harvest",
		GrimvaultTreeNode		= "harvest"
		}

	
	local ccObjectives = CColor.new(0, 0, 0, 1)
	local ccGivers = CColor.new(0, 0, 0, 1)
	local ccPath = CColor.new(0, 0, 0, 1)
	local ccHarvest = CColor.new(0, 0, 0, 1)
	local ccFarm = CColor.new(0, 0, 0, 1)
	
	self.tColor = {giver = ccGivers, objective = ccObjectives, path = ccPath, harvest = ccHarvest, farm = ccFarm}
	
	self.textNumberToString = {"CRB_Interface14", "CRB_InterfaceMedium_B", "CRB_InterfaceSmall", "CRB_InterfaceLarge"}
	self.textMarkerToString = {"Dots", "Arrow", "Flag"}
	self.textStringToNumber = {CRB_Interface14 = 1, CRB_InterfaceMedium_B = 2, CRB_InterfaceSmall = 3, CRB_InterfaceLarge = 4, Dots = 1, Arrow = 2, Flag = 3,}

	
	-- changes to these are saved and restored
	--TODO Add ability to Restore These settings
	self.tUserSettings = {}
	self.tUserSettings.lineCounts = {giver = 1, objective = 3, path = 1, harvest = 1, farm = 1}
	self.tUserSettings.wipe6 = false -- used to wipe old data from a point on
	self.tUserSettings.arrowTimer = 10
	self.tUserSettings.showLinesInCombat = true
	self.tUserSettings.showDrag = true
	self.tUserSettings.showGuideLineTracker = true
	self.tUserSettings.cnObjectives = {red = 1, green = 1, blue = 1, alpha = 1}
	self.tUserSettings.cnGivers = {red = 1, green = 1, blue = 0, alpha = 1}
	self.tUserSettings.cnPath = {red = 0, green = 0, blue = 1, alpha = 1}
	self.tUserSettings.cnHarvest = {red = 1, green = 0, blue = 1, alpha = 1}
	self.tUserSettings.cnFarm = {red = 0, green = .5, blue = 0, alpha = 1}
	self.tUserSettings.neverArrow = true
	self.tUserSettings.numPoints = 10
	self.tUserSettings.tradeSkill = true
	self.tUserSettings.countChallenge = false
	self.tUserSettings.textChoice = { GuideLineHeaderText = "CRB_Interface14", GuideLineObjectiveText = "CRB_Interface14" }
	self.tUserSettings.Marker = "Dots"

	return o
end

function GuideLine:Init()
	local bHasConfigureFunction = true
	local strConfigureButtonText = "GuideLine"
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end


-----------------------------------------------------------------------------------------------
-- GuideLine OnLoad
-----------------------------------------------------------------------------------------------
function GuideLine:OnLoad()
	Apollo.RegisterEventHandler("UnitCreated", "OnPreloadUnitCreated", self)
	self.xmlDoc = XmlDoc.CreateFromFile("GuideLine.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

function GuideLine:OnPreloadUnitCreated(unitNew)
	self.unitsToCheck[unitNew:GetId()] = unitNew
end


function GuideLine:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return
	end
	
	local tSavedData = self.tUserSettings

	tSavedData.nOL, tSavedData.nOT, tSavedData.nOR, tSavedData.nOB = self.wndContainer:GetAnchorOffsets()	
			
	return tSavedData
end

function GuideLine:OnRestore(eType, tSavedData)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return
	end

	if tSavedData ~= nil then
		--self.tUserSettings = tSavedData
		if tSavedData.wipe6 then 
			for idx, item in pairs(tSavedData) do
				self.tUserSettings[idx] = item
			end	
		end
			
	end
	self.tUserSettings.wipe6 = true
	self.tUserSettings.givers = nil
	self.tUserSettings.SAVEDUNITS = nil
	self.tUserSettings.lineCounts["farm"] = self.tUserSettings.lineCounts["farm"] or 1

end

-- 
function GuideLine:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndOptions = Apollo.LoadForm(self.xmlDoc, "GuideLineForm", nil, self)
		if self.wndOptions == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndOptions:Show(false, true)

		Apollo.RegisterSlashCommand("GuideLine", "OnGuideLineOn", self)

		self.timer = ApolloTimer.Create(self.tUserSettings.arrowTimer, true, "OnTimer", self)
		self.timer:Stop()
		
		if self.tUserSettings.arrowTimer ~= 0 then
			self.timer:Start()
		end
	
		

		-- Start Guidelines
		Apollo.RegisterEventHandler("QuestStateChanged", "OnQuestStateChanged", self)
		Apollo.RegisterEventHandler("QuestObjectiveUpdated", "OnQuestObjectiveUpdated", self)
		Apollo.RegisterEventHandler("GenericEvent_QuestLog_TrackBtnClicked", "OnSomethingChanged", self)
		Apollo.RegisterEventHandler("ChangeWorld", "OnWorldChanged", self)		
		Apollo.RemoveEventHandler("UnitCreated", self)
		Apollo.RegisterEventHandler("UnitCreated", "OnUnitCreated", self)
		Apollo.RegisterEventHandler("UnitDestroyed", "OnUnitDestroyed", self)
		Apollo.RegisterEventHandler("UnitActivationTypeChanged", "OnUnitChanged", self)
		Apollo.RegisterEventHandler("UnitGibbed", "OnUnitChanged", self)
		Apollo.RegisterEventHandler("UnitEnteredCombat", "OnEnterCombat", self)
		Apollo.RegisterEventHandler("MatchEntered", "OnEnterPvP", self)
		Apollo.RegisterEventHandler("PVPMatchFinished", "OnLeavePvP", self)
		Apollo.RegisterEventHandler("MatchExited", 	"OnLeavePvP", self)
		Apollo.RegisterEventHandler("GuideLine_Config", "OnGuideLineOn", self)

		self.wndContainer = Apollo.LoadForm(self.xmlDoc, "QuestContainer", "FixedHudStratum", self)
		if self.tUserSettings.nOL then
			self.wndContainer:SetAnchorOffsets(self.tUserSettings.nOL, self.tUserSettings.nOT, self.tUserSettings.nOR, self.tUserSettings.nOB)
		end
		self.wndTitleContainer = self.wndContainer:FindChild("TitleContainer")
		self.wndTitleText = self.wndTitleContainer:FindChild("TitleText")
		self.wndTitleGraphic = self.wndTitleContainer:FindChild("TitleText")
		self.wndObjectiveContainer = self.wndContainer:FindChild("ObjectiveContainer")
		self.wndSpacer = self.wndContainer:FindChild("MoveSpacer")
		self.wndSpacer:Show(self.tUserSettings.showDrag, true)
		self:bgFade()
		
		self.mapTextOptions = {
			{
				wnd = self.wndOptions:FindChild("btnHeaderText"),
				radio = "GuideLineHeaderText"
			},
			{
				wnd = self.wndOptions:FindChild("btnObjectiveText"),
				radio = "GuideLineObjectiveText"
			},
			{
				wnd = self.wndOptions:FindChild("btnMarkers"),
				radio = "GuideLineMarkers"
			}
		}
		
		for idx, wndDD in pairs(self.mapTextOptions) do
			wndDD.wnd:AttachWindow(wndDD.wnd:FindChild("ChoiceContainer"))
			wndDD.wnd:FindChild("ChoiceContainer"):Show(false)
		end

		for idx, parent in pairs(self.mapTextOptions) do
			if parent.wnd ~= nil and parent.radio ~= nil then
	
				local arBtns = parent.wnd:FindChild("ChoiceContainer"):GetChildren()
	
				for idxBtn = 1, #arBtns do
					arBtns[idxBtn]:SetCheck(false)
				end
				
				local idxBtn = self.textStringToNumber[self.tUserSettings.textChoice[parent.radio]]
				local idxBtnMarker = self.textStringToNumber[self.tUserSettings.Marker]			
				if idxBtn ~= nil then
				
					self.wndOptions:SetRadioSel(parent.radio, idxBtn )
					arBtns[idxBtn]:SetCheck(true)
					parent.wnd:SetFont(self.tUserSettings.textChoice[parent.radio])
				else
					self.wndOptions:SetRadioSel(parent.radio, idxBtnMarker )
					arBtns[idxBtnMarker]:SetCheck(true)
				end
				
			end
		end

		
		self.wndContainer:Show(self.tUserSettings.showGuideLineTracker, true)
		
		self.drawTimer = ApolloTimer.Create(0.01, true, "OnDrawTimer", self)
		
		self.delayStartTime = ApolloTimer.Create(0.5, true, "OnDelayedStart", self)
		
		self.questUpdateTimer = ApolloTimer.Create(5, true, "OnQuestUpdateTimer", self)
		
	end
end


function GuideLine:OnDelayedStart()
	self.player = GameLib.GetPlayerUnit()
	if not self.player then return end
	
	self.delayStartTime:Stop()
	
	self:UpdateCarbineClosest(false)
	self:OnTimer()

	local nPath = PlayerPathLib.GetPlayerPathType()
	if nPath == 0 then
		self.pathName = "Soldier"
	elseif nPath == 1 then
		self.pathName = "Settler"
	elseif nPath == 2 then
		self.pathName = "Scientist"
	elseif nPath == 3 then
		self.pathName = "Explorer" 
	end
	
	self:UpdateLineColors()
	
	self.shouldUpdatePossibleLines = true
	
	
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "GuideLine", {"GuideLine_Config", "", "CRB_Basekit:kitIcon_Holo_Exclamation"})

end

function GuideLine:OnWorldChanged()
	self.player = nil
	self.shouldUpdateUnits = true
	self.shouldUpdatePossibleLines = true
	
end



function GuideLine:OnSomethingChanged()
	
	self.shouldUpdateUnits = true	
	self.shouldUpdatePossibleLines = true

	
end

function GuideLine:OnQuestStateChanged(que)

	self:UpdateCarbineClosest(false)
	self:ArrowIfCan()
	self.shouldUpdateUnits = true	
	self.shouldUpdatePossibleLines = true

	
end

function GuideLine:OnQuestObjectiveUpdated(que, objNum)
	if que == self.questCarbine then
		self:UpdateQuestCurrent(que)
		if self.lastObjectiveNum ~= objNum then
			self.lastObjectiveNum = objNum
			self:ArrowIfCan()
		end
	end
	self.shouldUpdateUnits = true
	self.shouldUpdatePossibleLines = true

end

function GuideLine:OnEnterPvP()
	self.wndContainer:Show(false, true)
	self.pvp = true

end

function GuideLine:OnLeavePvP()
	self.wndContainer:Show(self.tUserSettings.showGuideLineTracker, true)
	self.pvp = false
end


function GuideLine:OnEnterCombat(unit, inCombat)
	if unit == GameLib.GetPlayerUnit() then
		self.inCombat = inCombat
	end
end

function GuideLine:OnUnitCreated(unit)
	-- Hopefully filter out 'used Item quest givers that can't be used atm'
	if not unit then return end
		
	self.unitsToCheck[unit:GetId()] = unit
		
	
end

function GuideLine:OnUnitChanged(unit)
	self:CheckTracking(unit)		
	self.shouldUpdatePossibleLines = true
end

function GuideLine:OnUnitDestroyed(unit)
	local id = unit:GetId()
	local tType = self.tUnits[id]
	
	if tType then
		self.tTracked[tType][id] = nil
		self.tUnits[id] = nil
		self.shouldUpdatePossibleLines = true
	end
		
end



function GuideLine:OnQuestUpdateTimer()
	local lastQue = self.questCarbine
	self:UpdateCarbineClosest(false)
	if lastQue ~= self.questCarbine then
		self:ArrowIfCan()
	end
end




function GuideLine:OnTimer()
	if self.lineUsedCount < 1 then
		self:ArrowIfCan()
	end
end

function GuideLine:UpdateCarbineClosest(force)

	local epis = QuestLib.GetTrackedEpisodes(true)
	if epis[1] then
		local quests = epis[1]:GetTrackedQuests(0, true)
		local que = quests[1]
		if que then
			if self.questCarbine == que and not force then
				-- do anything here?
			else
				self.questCarbine = que
				self:UpdateQuestCurrent(que)
			end
			return 
		end
	end
	
	self.quest = nil
	self.wndTitleText:SetText("No Quests Tracked - Find a quest or Track one in your Quest Log")
	self.wndTitleText:SetHeightToContentHeight()
	self.wndTitleGraphic:SetHeightToContentHeight()
	local h = self.wndTitleText:GetHeight()+20
	--self.wndTitleContainer:SetAnchorOffsets(0, 20, 0, h)
	for idx, wnd in ipairs(self.wndObjectivesList) do
		wnd:Destroy()
	end

	
end


	

function GuideLine:ArrowIfCan()
	if self.tUserSettings.neverArrow then return end
	if self.questCarbine then
		local reg = self.questCarbine:GetMapRegions()
		
		if #reg > 0 and reg[1].tIndicator then
		-- added to hide trade skill window from triggering
			if self.questCarbine:GetSubType() ~= 113 or not self.tUserSettings.tradeSkill then
				self.questCarbine:ShowHintArrow()
			end
		end

	end
		
end

function GuideLine:UpdateQuestCurrent(quest)
	-- todo handle timers, but really we should figure out if any quest has a timer and prioritize that over distance shouldn't we?  bleh
	-- first up title

	
	local strTitle = quest:GetTitle()
	local eQuestState = quest:GetState()
	if eQuestState == Quest.QuestState_Botched then
		strTitle = String_GetWeaselString(Apollo.GetString("QuestTracker_Failed"), strTitle)
	elseif eQuestState == Quest.QuestState_Achieved then
		strTitle = String_GetWeaselString(Apollo.GetString("QuestTracker_Complete"), strTitle)
	end
	strTitle = String_GetWeaselString(Apollo.GetString("QuestTracker_ConDisplay"), strTitle, quest:GetConLevel())
	self.wndTitleText:SetFont(self.tUserSettings.textChoice["GuideLineHeaderText"])
	self.wndTitleText:SetText(strTitle)
	self.wndTitleText:SetHeightToContentHeight()
	
	for idx, wnd in ipairs(self.wndObjectivesList) do
		wnd:Destroy()
	end
	
	self.wndObjectivesList = {}
	local qH = 0
	if eQuestState == Quest.QuestState_Achieved then
		local strResult = quest:GetCompletionObjectiveShortText()
		if not strResult or string.len(strResult) <= 0 then
			strResult = quest:GetCompletionObjectiveText()
		end
		local wnd = Apollo.LoadForm(self.xmlDoc, "Objective", self.wndObjectiveContainer, self)
		table.insert(self.wndObjectivesList, wnd)
		wnd:SetFont(self.tUserSettings.textChoice["GuideLineObjectiveText"])
		wnd:SetText(strResult)
	else
		for idObjective, tObjective in ipairs(quest:GetVisibleObjectiveData()) do
			if tObjective.nCompleted < tObjective.nNeeded then
				local wnd = Apollo.LoadForm(self.xmlDoc, "Objective", self.wndObjectiveContainer, self)
				table.insert(self.wndObjectivesList, wnd)
				local strResult = quest:GetObjectiveShortDescription(tObjective.nIndex)
				if not strResult or string.len(strResult) <= 0 then
					strResult = tObjective.strDescription
				end
				
				wnd:FindChild("BtnObjective"):SetData({q = quest, i = tObjective.nIndex})
				
				local pre = ""
				if tObjective and not tObjective.bIsRequired then
					pre = Apollo.GetString("QuestLog_Optional")
				end
				if tObjective.nNeeded > 1 and quest:DisplayObjectiveProgressBar(tObjective.nIndex) then
					pre = pre .. String_GetWeaselString(Apollo.GetString("QuestTracker_PercentComplete"), tObjective.nCompleted)
				elseif tObjective.nNeeded > 1 then 
					pre = pre .. String_GetWeaselString(Apollo.GetString("QuestTracker_ValueComplete"), tObjective.nCompleted, tObjective.nNeeded)
				end
				
				if quest:IsObjectiveTimed(tObjective.nIndex) then
					pre = pre .. math.floor(quest:GetObjectiveTimeRemaining(tObjective.nIndex) / 1000) .. "s"
				end
				
				strResult = pre .. " " .. strResult

				wnd:SetFont(self.tUserSettings.textChoice["GuideLineObjectiveText"])
				wnd:SetText(strResult)
				wnd:SetHeightToContentHeight()
				qH = qH + wnd:GetHeight()
			end
		end
	end
	
	local lf = {self.wndContainer:GetAnchorOffsets()}
	-- left   = 1
	-- top    = 2
	-- right  = 3
	-- bottom = 4
	
	qH = qH+120+lf[2]
	self.wndContainer:SetAnchorOffsets(lf[1],lf[2],lf[3],qH)
	self.wndObjectiveContainer:ArrangeChildrenVert()
 
end


function GuideLine:CheckTracking(unit)
	
	if not self.player then return end
	if not unit or not unit:IsValid() then return end
	local shouldTrack = "none"

	local tRewardInfo = nil
	local reward = false
	tRewardInfo = unit:GetRewardInfo()
	if tRewardInfo and type(tRewardInfo) == "table" then
		local count = #tRewardInfo
		for idx = 1, count do
			local type = tRewardInfo[idx].strType
			if type == "Quest" then
				shouldTrack = "objective"
				reward = true
			elseif type == self.pathName then
				if tRewardInfo[idx].pmMission then
					shouldTrack = "path"
				end
			elseif type == "PublicEvent" then
				if tRewardInfo[idx].peoObjective then
					if tRewardInfo[idx].peoObjective:GetEvent():GetName() ~= nil then
						shouldTrack = "objective"
						reward = true
					end
				end
			elseif self.tUserSettings.countChallenge and type == "Challenge" then
				for index, clgCurr in pairs(ChallengesLib.GetActiveChallengeList()) do
					if tRewardInfo[idx].idChallenge == clgCurr:GetId() and clgCurr:IsActivated() and not clgCurr:IsInCooldown() and not clgCurr:ShouldCollectReward() then
						shouldTrack = "objective"
						break
					end
				end		
			end
			
		end
	end
	
	
	local tActivation = unit:GetActivationState()
			
	if tActivation.QuestNew ~= nil or tActivation.QuestNewMain ~= nil then
		shouldTrack = "giver" 
	elseif tActivation.QuestNewRepeatable ~= nil then
		shouldTrack = "giver"
	elseif tActivation.QuestReward ~= nil then
		shouldTrack = "objective"
	elseif tActivation.Datacube ~= nil then
		if self.pathName == "Scientist" then
			shouldTrack = "path"
		end
	elseif self.pathName == "Settler" and tActivation.Collect ~= nil and unit:GetAffiliationName() ~= "" then
		shouldTrack = "path"
	elseif self.pathName == "Settler" and tActivation.SettlerImprovement ~= nil then
		shouldTrack = "path"
	elseif self.pathName == "Explorer" and tActivation.ExplorerInterest ~= nil then
		shouldTrack = "path"
	end
		
	
	if tActivation.Busy ~= nil then
		shouldTrack = "none"
	end

	if shouldTrack == "none" then
		if unit:CanBeHarvestedBy(self.player) then
			local minimapMarkers = unit:GetMiniMapMarkers()
			for i, marker in ipairs(minimapMarkers) do
				if self.miniMapMarker[marker] then
					shouldTrack = self.miniMapMarker[marker]
				end
			end
		end
	end
	
	if shouldTrack == "none" then
		if unit:GetType() == "Pickup" then
			local pName = self.player:GetName();
			if string.sub(unit:GetName(), 1, string.len(pName)) == pName then
				shouldTrack = "objective"
			end
		end
	end	
			
	if unit:IsDead() then
		shouldTrack = "none"
	end
		
	
	local id = unit:GetId()
	if self.tUnits[id] then
		local old = self.tUnits[id]
		if old ~= shouldTrack then
			self.tTracked[old][id] = nil
		end	
	end
	self.tUnits[id] = shouldTrack
	self.tTracked[shouldTrack][id] = {unit = unit}
	
end

function GuideLine:StopUsingMarkerSet(id)
	local markerSet = self.markerSets[id]
	if markerSet then
		for i = 1, self.numPoints do
			if #markerSet >= i then
				markerSet[i]:Show(false, true)
			end
		end
	end
	self.markerSets[id] = nil
	if markerSet then
		table.insert(self.unusedMarkerSets, markerSet)
	end

end

--------------------
-- Draw Line
--------------------
function GuideLine:DrawLineTo(id, targetVec, dist, which)
	
	local color = self.tColor[which]
	
	local markerSet = nil
	if self.markerSets[id] then
		markerSet = self.markerSets[id] 
	end

	if markerSet == nil or self.marker[id] ~= self.tUserSettings.Marker or self.numPoints ~= self.tUserSettings.numPoints then
	
		if (self.marker[id] ~= self.tUserSettings.Marker or self.numPoints ~= self.tUserSettings.numPoints) and markerSet ~= nil then
		--Clean up any markers 
			local m = self.markerSets[id]
			for i = 1, self.numPoints do
				if #m >= i then
					m[i]:Show(false, true)
				end
			end
			
		end
		
		self.marker[id] = self.tUserSettings.Marker
		self.numPoints = self.tUserSettings.numPoints
		
		markerSet = {}
		for i = 1, self.numPoints  do
			if self.tUserSettings.Marker == "Dots" then
				markerSet[i] = Apollo.LoadForm("GuideLine.xml", "GuideLineMarker", "InWorldHudStratum", self)
			elseif self.tUserSettings.Marker == "Arrow" then
				markerSet[i] = Apollo.LoadForm("GuideLine.xml", "GuideLineArrow", "InWorldHudStratum", self)
			else
				markerSet[i] = Apollo.LoadForm("GuideLine.xml", "GuideLineFlag", "InWorldHudStratum", self)
			end
		end
		self.markerSets[id] = markerSet
	end
	
	for i = 1, self.numPoints  do
	--self.wndOptions:FindChild("Debug"):SetText(color);
		local fraction = i/self.numPoints 
		if #markerSet >= i then
			markerSet[i]:SetBGColor(color)
			markerSet[i]:SetWorldLocation(Vector3.InterpolateLinear(self.playerVector, targetVec, fraction))
			if markerSet[i]:IsOnScreen() then
				markerSet[i]:Show(true, true)
			else
				markerSet[i]:Show(false, true)
			end
		end
	end
	
	self.tUnitsDrawn[id] = 1	
end



function GuideLine:OnDrawTimer()

	if not self.player then
		self.player = GameLib.GetPlayerUnit()	
		return
	end
	
	for id, unit in pairs(self.unitsToCheck) do
		self:CheckTracking(unit)
		self.unitsToCheck[id] = nil
	end
	
	
	self.curPosition = self.player:GetPosition()
	if self.curPosition == nil then return end
	if self.lastPosition == nil then
		self.lastPosition = self.curPosition
	end
	
	local moving = true
	if  math.abs(self.lastPosition.x - self.curPosition.x) < 0.01 and 
		math.abs(self.lastPosition.y - self.curPosition.y) < 0.01 and 
		math.abs(self.lastPosition.z - self.curPosition.z) < 0.01 then	
		moving = false
	end
	self.lastPosition = self.curPosition
	
	if moving then
		self.shouldUpdatePossibleLines = true
	end
	
	if self.time and Time.SecondsElapsed(self.time) > 5 then
		self.shouldUpdateUnits = true
		self.shouldUpdatePossibleLines = true
	end

	if self.shouldUpdateUnits then
		self.shouldUpdateUnits = false

		for id, trackingType in pairs(self.tUnits) do
			self:CheckTracking(self.tTracked[trackingType][id]["unit"])
		end		
	end
	

	if self.shouldUpdatePossibleLines then
		self.shouldUpdatePossibleLines = false
		self.time = Time.Now()
				
		self.playerVector = Vector3.New(self.curPosition.x, self.curPosition.y, self.curPosition.z)
		
		self.haveNoDist = false
		self.tUnitsDrawn = {}
		
		self.lineUsedCount = 0
		self:UpdateTrackingLines("objective")		
		self:UpdateTrackingLines("giver")
		self:UpdateTrackingLines("harvest")
		self:UpdateTrackingLines("path")
		self:UpdateTrackingLines("farm")
			
		for id, set in pairs(self.markerSets) do
			if not self.tUnitsDrawn[id] then
				self:StopUsingMarkerSet(id)
			end
		end
		
		
	end
end

function GuideLine:UpdateDistances(which)
	for id, blob in pairs(self.tTracked[which]) do
		local targetPos = blob["unit"]:GetPosition()
		blob["targetVec"] = Vector3.New(targetPos.x, targetPos.y, targetPos.z)
		blob["dist"] = (self.playerVector - blob["targetVec"]):Length()
		blob["id"] = id
		if not blob["dist"] then self.haveNoDist = true end
	end
end

function GuideLine:UpdateTrackingLines(which)

	if self.tUserSettings.lineCounts[which] > 0 then
		
		self:UpdateDistances(which)
		
		local blobList = {}
		for id, unitBlob in pairs(self.tTracked[which]) do
			table.insert(blobList, unitBlob)
		end
		
		if not self.player.IsInCCState(Unit.CodeEnumCCState.Blind) and not self.haveNoDist then
			table.sort(blobList, function(a, b) return a["dist"] < b["dist"] end)
			
			for i = 1, self.tUserSettings.lineCounts[which] do
				if blobList[i] then
					if which == "objective" then self.lineUsedCount = self.lineUsedCount + 1 end
					self:DrawLineTo(blobList[i]["id"], blobList[i]["targetVec"], blobList[i]["dist"], which)
				end	
			end
		end

	end

end

function GuideLine:UpdateLineColors()
	self.tColor["objective"] = CColor.new(self.tUserSettings.cnObjectives["red"], self.tUserSettings.cnObjectives["green"], self.tUserSettings.cnObjectives["blue"], self.tUserSettings.cnObjectives["alpha"])
	self.tColor["giver"] = CColor.new(self.tUserSettings.cnGivers["red"], self.tUserSettings.cnGivers["green"], self.tUserSettings.cnGivers["blue"], self.tUserSettings.cnGivers["alpha"])
	self.tColor["path"] = CColor.new(self.tUserSettings.cnPath["red"], self.tUserSettings.cnPath["green"], self.tUserSettings.cnPath["blue"], self.tUserSettings.cnPath["alpha"])
	self.tColor["harvest"] = CColor.new(self.tUserSettings.cnHarvest["red"], self.tUserSettings.cnHarvest["green"], self.tUserSettings.cnHarvest["blue"], self.tUserSettings.cnHarvest["alpha"])
	self.tColor["farm"] = CColor.new(self.tUserSettings.cnFarm["red"], self.tUserSettings.cnFarm["green"], self.tUserSettings.cnFarm["blue"], self.tUserSettings.cnFarm["alpha"])

end


function GuideLine:OnQuestCloseBtn(wndHandler, wndControl)
	if self.questCarbine == nil then return end
	self.questCarbine:SetActiveQuest(false)
	if self.questCarbine:GetState() == Quest.QuestState_Botched then
		self.questCarbine:Abandon()
	else
		self.questCarbine:ToggleTracked()
	end
	self:UpdateCarbineClosest(false)
end

function GuideLine:OnGuideLineOn()

	self.wndOptions:FindChild("inputObjectiveCount"):SetText(tostring(self.tUserSettings.lineCounts["objective"]))
	self.wndOptions:FindChild("inputGiverCount"):SetText(tostring(self.tUserSettings.lineCounts["giver"]))
	self.wndOptions:FindChild("inputPathCount"):SetText(tostring(self.tUserSettings.lineCounts["path"]))
	self.wndOptions:FindChild("inputHarvestCount"):SetText(tostring(self.tUserSettings.lineCounts["harvest"]))
	self.wndOptions:FindChild("inputFarmCount"):SetText(tostring(self.tUserSettings.lineCounts["farm"]))
	self.wndOptions:FindChild("inputNumPoints"):SetText(tostring(self.tUserSettings.numPoints))
	
	--self.wndOptions:FindChild("btnQuestGiversAlways"):SetCheck(self.tUserSettings.showPriorityLines)
	self.wndOptions:FindChild("inputArrowTimer"):SetText(tostring(self.tUserSettings.arrowTimer))
	
	self.wndOptions:FindChild("btnshowDrag"):SetCheck(self.tUserSettings.showDrag)
	self.wndOptions:FindChild("btnshowGuideLineTracker"):SetCheck(self.tUserSettings.showGuideLineTracker)
	self.wndOptions:FindChild("btnNeverArrow"):SetCheck(self.tUserSettings.neverArrow)
	self.wndOptions:FindChild("btnTradeSkill"):SetCheck(self.tUserSettings.tradeSkill)
	self.wndOptions:FindChild("btnChallenges"):SetCheck(self.tUserSettings.countChallenge)

	self.wndOptions:FindChild("cnObjectivesRed"):SetValue(self.tUserSettings.cnObjectives["red"])
	self.wndOptions:FindChild("cnObjsRedTxt"):SetText(strRound(self.tUserSettings.cnObjectives["red"],6));
	
	self.wndOptions:FindChild("cnObjectivesGreen"):SetValue(self.tUserSettings.cnObjectives["green"])
	self.wndOptions:FindChild("cnObjsGreenTxt"):SetText(strRound(self.tUserSettings.cnObjectives["green"],6));
	self.wndOptions:FindChild("cnObjectivesBlue"):SetValue(self.tUserSettings.cnObjectives["blue"])
	self.wndOptions:FindChild("cnObjsBlueTxt"):SetText(strRound(self.tUserSettings.cnObjectives["blue"],6));
	self.wndOptions:FindChild("cnObjectivesAlpha"):SetValue(self.tUserSettings.cnObjectives["alpha"])
	self.wndOptions:FindChild("cnObjsAlphaTxt"):SetText(strRound(self.tUserSettings.cnObjectives["alpha"],6));
	
	self.wndOptions:FindChild("cnGiversRed"):SetValue(self.tUserSettings.cnGivers["red"])
	self.wndOptions:FindChild("cnGivRedTxt"):SetText(strRound(self.tUserSettings.cnGivers["red"],6));
	self.wndOptions:FindChild("cnGiversGreen"):SetValue(self.tUserSettings.cnGivers["green"])
	self.wndOptions:FindChild("cnGivGreenTxt"):SetText(strRound(self.tUserSettings.cnGivers["green"],6));
	self.wndOptions:FindChild("cnGiversBlue"):SetValue(self.tUserSettings.cnGivers["blue"])
	self.wndOptions:FindChild("cnGivBlueTxt"):SetText(strRound(self.tUserSettings.cnGivers["blue"],6));
	self.wndOptions:FindChild("cnGiversAlpha"):SetValue(self.tUserSettings.cnGivers["alpha"])
	self.wndOptions:FindChild("cnGivAlphaTxt"):SetText(strRound(self.tUserSettings.cnGivers["alpha"],6));
	
	self.wndOptions:FindChild("cnPathRed"):SetValue(self.tUserSettings.cnPath["red"])
	self.wndOptions:FindChild("cnPathGreen"):SetValue(self.tUserSettings.cnPath["green"])
	self.wndOptions:FindChild("cnPathBlue"):SetValue(self.tUserSettings.cnPath["blue"])
	self.wndOptions:FindChild("cnPathAlpha"):SetValue(self.tUserSettings.cnPath["alpha"])
	self.wndOptions:FindChild("cnPathRedTxt"):SetText(strRound(self.tUserSettings.cnPath["red"],6));
	self.wndOptions:FindChild("cnPathGreenTxt"):SetText(strRound(self.tUserSettings.cnPath["green"],6));
	self.wndOptions:FindChild("cnPathBlueTxt"):SetText(strRound(self.tUserSettings.cnPath["blue"],6));
	self.wndOptions:FindChild("cnPathAlphaTxt"):SetText(strRound(self.tUserSettings.cnPath["alpha"],6));
	
	self.wndOptions:FindChild("cnHarvestRed"):SetValue(self.tUserSettings.cnHarvest["red"])
	self.wndOptions:FindChild("cnHarvestGreen"):SetValue(self.tUserSettings.cnHarvest["green"])
	self.wndOptions:FindChild("cnHarvestBlue"):SetValue(self.tUserSettings.cnHarvest["blue"])
	self.wndOptions:FindChild("cnHarvestAlpha"):SetValue(self.tUserSettings.cnHarvest["alpha"])
	self.wndOptions:FindChild("cnHarvestRedTxt"):SetText(strRound(self.tUserSettings.cnHarvest["red"],6));
	self.wndOptions:FindChild("cnHarvestGreenTxt"):SetText(strRound(self.tUserSettings.cnHarvest["green"],6));
	self.wndOptions:FindChild("cnHarvestBlueTxt"):SetText(strRound(self.tUserSettings.cnHarvest["blue"],6));
	self.wndOptions:FindChild("cnHarvestAlphaTxt"):SetText(strRound(self.tUserSettings.cnHarvest["alpha"],6));
		
	self.wndOptions:FindChild("cnFarmRed"):SetValue(self.tUserSettings.cnFarm["red"])
	self.wndOptions:FindChild("cnFarmGreen"):SetValue(self.tUserSettings.cnFarm["green"])
	self.wndOptions:FindChild("cnFarmBlue"):SetValue(self.tUserSettings.cnFarm["blue"])
	self.wndOptions:FindChild("cnFarmAlpha"):SetValue(self.tUserSettings.cnFarm["alpha"])
	self.wndOptions:FindChild("cnFarmRedTxt"):SetText(strRound(self.tUserSettings.cnFarm["red"],6));
	self.wndOptions:FindChild("cnFarmGreenTxt"):SetText(strRound(self.tUserSettings.cnFarm["green"],6));
	self.wndOptions:FindChild("cnFarmBlueTxt"):SetText(strRound(self.tUserSettings.cnFarm["blue"],6));
	self.wndOptions:FindChild("cnFarmAlphaTxt"):SetText(strRound(self.tUserSettings.cnFarm["alpha"],6));
	
	
	self.wndOptions:FindChild("ccObjectivesShow"):SetBGColor(self.tColor["objective"])
	self.wndOptions:FindChild("ccGiversShow"):SetBGColor(self.tColor["giver"])
	self.wndOptions:FindChild("ccPathShow"):SetBGColor(self.tColor["path"])
	self.wndOptions:FindChild("ccHarvestShow"):SetBGColor(self.tColor["harvest"])
	self.wndOptions:FindChild("ccFarmShow"):SetBGColor(self.tColor["farm"])

	self.wndOptions:Invoke() -- show the window
end

function GuideLine:OnConfigure()
	self:OnGuideLineOn()
end



function GuideLine:OnOK()

	self.tUserSettings.lineCounts["objective"] = tonumber(self.wndOptions:FindChild("inputObjectiveCount"):GetText()) or 0
	self.tUserSettings.lineCounts["giver"] = tonumber(self.wndOptions:FindChild("inputGiverCount"):GetText()) or 0
	self.tUserSettings.lineCounts["path"] = tonumber(self.wndOptions:FindChild("inputPathCount"):GetText()) or 0
	self.tUserSettings.lineCounts["harvest"] = tonumber(self.wndOptions:FindChild("inputHarvestCount"):GetText()) or 0
	self.tUserSettings.lineCounts["farm"] = tonumber(self.wndOptions:FindChild("inputFarmCount"):GetText()) or 0
	self.tUserSettings.numPoints = tonumber(self.wndOptions:FindChild("inputNumPoints"):GetText()) or 1

	self.tUserSettings.arrowTimer = tonumber(self.wndOptions:FindChild("inputArrowTimer"):GetText()) or 0
	self.tUserSettings.showDrag = self.wndOptions:FindChild("btnshowDrag"):IsChecked()
	self.tUserSettings.showGuideLineTracker = self.wndOptions:FindChild("btnshowGuideLineTracker"):IsChecked()
	self.tUserSettings.neverArrow = self.wndOptions:FindChild("btnNeverArrow"):IsChecked()
	self.tUserSettings.tradeSkill = self.wndOptions:FindChild("btnTradeSkill"):IsChecked()
	self.tUserSettings.countChallenge = self.wndOptions:FindChild("btnChallenges"):IsChecked()

	-- valid input or default
	self.tUserSettings.arrowTimer = self.tUserSettings.arrowTimer or 10
	self.tUserSettings.lineCount = self.tUserSettings.lineCount or 3

	-- actions with new stuff
	self.wndSpacer:Show(self.tUserSettings.showDrag, true)
	
	self.timer:Set(self.tUserSettings.arrowTimer)
	self.timer:Stop()
		if self.tUserSettings.arrowTimer ~= 0 then
		self.timer:Start()
	end
	
	self.wndContainer:Show(self.tUserSettings.showGuideLineTracker, true)
		
	self:UpdateCarbineClosest(true)
	self.wndOptions:Close() -- hide the window
end

-- when the Cancel button is clicked
function GuideLine:OnCancel()
	self.wndOptions:Close() -- hide the window
end


function GuideLine:onColorOptionsChanged( wndHandler, wndControl, strText )
	-- I know what you're gonna think, and I don't wanna talk about it.  I don't.  Don't bring it up.  Just leave me to my shame.
	
	if wndHandler:GetName() == "cnObjectivesRed" then
		local n = tonumber(wndHandler:GetText())
		n = GuideLine.ValidateCN(n)
		self.tUserSettings.cnObjectives["red"] = n
	elseif wndHandler:GetName() == "cnObjectivesGreen" then
		local n = tonumber(wndHandler:GetText())
		n = GuideLine.ValidateCN(n)
		self.tUserSettings.cnObjectives["green"] = n
	elseif wndHandler:GetName() == "cnObjectivesBlue" then
		local n = tonumber(wndHandler:GetText())
		n = GuideLine.ValidateCN(n)
		self.tUserSettings.cnObjectives["blue"] = n
	elseif wndHandler:GetName() == "cnObjectivesAlpha" then
		local n = tonumber(wndHandler:GetText())
		n = GuideLine.ValidateCN(n)
		self.tUserSettings.cnObjectives["alpha"] = n
	
	elseif wndHandler:GetName() == "cnGiversRed" then
		local n = tonumber(wndHandler:GetText())
		n = GuideLine.ValidateCN(n)
		self.tUserSettings.cnGivers["red"] = n
	elseif wndHandler:GetName() == "cnGiversGreen" then
		local n = tonumber(wndHandler:GetText())
		n = GuideLine.ValidateCN(n)
		self.tUserSettings.cnGivers["green"] = n
	elseif wndHandler:GetName() == "cnGiversBlue" then
		local n = tonumber(wndHandler:GetText())
		n = GuideLine.ValidateCN(n)
		self.tUserSettings.cnGivers["blue"] = n
	elseif wndHandler:GetName() == "cnGiversAlpha" then
		local n = tonumber(wndHandler:GetText())
		n = GuideLine.ValidateCN(n)
		self.tUserSettings.cnGivers["alpha"] = n
	
	elseif wndHandler:GetName() == "cnPathRed" then
		local n = tonumber(wndHandler:GetText())
		n = GuideLine.ValidateCN(n)
		self.tUserSettings.cnPath["red"] = n
	elseif wndHandler:GetName() == "cnPathGreen" then
		local n = tonumber(wndHandler:GetText())
		n = GuideLine.ValidateCN(n)
		self.tUserSettings.cnPath["green"] = n
	elseif wndHandler:GetName() == "cnPathBlue" then
		local n = tonumber(wndHandler:GetText())
		n = GuideLine.ValidateCN(n)
		self.tUserSettings.cnPath["blue"] = n
	elseif wndHandler:GetName() == "cnPathAlpha" then
		local n = tonumber(wndHandler:GetText())
		n = GuideLine.ValidateCN(n)
		self.tUserSettings.cnPath["alpha"] = n
		
	elseif wndHandler:GetName() == "cnHarvestRed" then
		local n = tonumber(wndHandler:GetText())
		n = GuideLine.ValidateCN(n)
		self.tUserSettings.cnHarvest["red"] = n
	elseif wndHandler:GetName() == "cnHarvestGreen" then
		local n = tonumber(wndHandler:GetText())
		n = GuideLine.ValidateCN(n)
		self.tUserSettings.cnHarvest["green"] = n
	elseif wndHandler:GetName() == "cnHarvestBlue" then
		local n = tonumber(wndHandler:GetText())
		n = GuideLine.ValidateCN(n)
		self.tUserSettings.cnHarvest["blue"] = n
	elseif wndHandler:GetName() == "cnHarvestAlpha" then
		local n = tonumber(wndHandler:GetText())
		n = GuideLine.ValidateCN(n)
		self.tUserSettings.cnHarvest["alpha"] = n

		
	elseif wndHandler:GetName() == "cnFarmRed" then
		local n = tonumber(wndHandler:GetText())
		n = GuideLine.ValidateCN(n)
		self.tUserSettings.cnFarm["red"] = n
	elseif wndHandler:GetName() == "cnFarmGreen" then
		local n = tonumber(wndHandler:GetText())
		n = GuideLine.ValidateCN(n)
		self.tUserSettings.cnFarm["green"] = n
	elseif wndHandler:GetName() == "cnFarmBlue" then
		local n = tonumber(wndHandler:GetText())
		n = GuideLine.ValidateCN(n)
		self.tUserSettings.cnFarm["blue"] = n
	elseif wndHandler:GetName() == "cnFarmAlpha" then
		local n = tonumber(wndHandler:GetText())
		n = GuideLine.ValidateCN(n)
		self.tUserSettings.cnFarm["alpha"] = n

	end
	
	
	self:UpdateLineColors()
	self.wndOptions:FindChild("ccObjectivesShow"):SetBGColor(self.tColor["objective"])
	self.wndOptions:FindChild("ccGiversShow"):SetBGColor(self.tColor["giver"])
	self.wndOptions:FindChild("ccPathShow"):SetBGColor(self.tColor["path"])
	self.wndOptions:FindChild("ccHarvestShow"):SetBGColor(self.tColor["harvest"])
	self.wndOptions:FindChild("ccFarmShow"):SetBGColor(self.tColor["farm"])


end

function GuideLine.ValidateCN(n)
	if not n then n = 0
	elseif n > 1 then n = 1
	elseif n < 0 then n = 0
	end
	return n
end

---------------------------------------------------------------------------------------------------
-- Objective Functions
---------------------------------------------------------------------------------------------------

function GuideLine:onObjectiveBtn( wndHandler, wndControl, eMouseButton )
	local d = wndHandler:GetData()
	if d and d.q and d.i then
		d.q:ShowHintArrow(d.i)
	end
	
end

---------------------------------------------------------------------------------------------------
-- GuideLineForm Functions
---------------------------------------------------------------------------------------------------

function GuideLine:OnGuideLineRadio( wndHandler, wndControl, eMouseButton )
	for idx, wndDD in pairs(self.mapTextOptions) do
		if wndDD.wnd == wndControl:GetParent():GetParent() then
			local btnidx =  wndControl:GetParent():GetRadioSel(wndDD.radio)
			local font = self.textNumberToString[btnidx]
			self.tUserSettings.textChoice[wndDD.radio] = font
			wndControl:GetParent():GetParent():SetFont(font)
			break
		end
	end
	
	wndControl:GetParent():Close()
	self:UpdateCarbineClosest(true)
end

function GuideLine:OnGuideLineMarker( wndHandler, wndControl, eMouseButton )
	for idx, wndDD in pairs(self.mapTextOptions) do
		if wndDD.wnd == wndControl:GetParent():GetParent() then
			local btnidx =  wndControl:GetParent():GetRadioSel(wndDD.radio)
			local Marker = self.textMarkerToString[btnidx]
			self.tUserSettings.Marker = Marker 
			break
		end
	end
	
	wndControl:GetParent():Close()
	self:OnSomethingChanged()
end


function GuideLine:bgVisable()
	local bg = self.wndContainer:FindChild("Graphics")
	bg:SetOpacity(.9)
end

function GuideLine:bgFade()
	local bg = self.wndContainer:FindChild("Graphics")
	bg:SetOpacity(.7)
end

function GuideLine:onColorSliderChanged( wndHandler, wndControl, fNewValue, fOldValue )
	-- I know what you're gonna think, and I don't wanna talk about it.  I don't.  Don't bring it up.  Just leave me to my shame.
	-- was able to cut the code in 1/2 by moving out the n and adding n2 functions.
	
	local n = tonumber(fNewValue)
	n = GuideLine.ValidateCN(n)
	local n2 = strRound(n,6)
	
	if wndHandler:GetName() == "cnObjectivesRed" then
		self.tUserSettings.cnObjectives["red"] = n
		self.wndOptions:FindChild("cnObjsRedTxt"):SetText(n2);
	elseif wndHandler:GetName() == "cnObjectivesGreen" then
		self.tUserSettings.cnObjectives["green"] = n
		self.wndOptions:FindChild("cnObjsGreenTxt"):SetText(n2);
	elseif wndHandler:GetName() == "cnObjectivesBlue" then
		self.tUserSettings.cnObjectives["blue"] = n
		self.wndOptions:FindChild("cnObjsBlueTxt"):SetText(n2);
	elseif wndHandler:GetName() == "cnObjectivesAlpha" then
		self.tUserSettings.cnObjectives["alpha"] = n
		self.wndOptions:FindChild("cnObjsAlphaTxt"):SetText(n2);
	elseif wndHandler:GetName() == "cnGiversRed" then
		self.tUserSettings.cnGivers["red"] = n
		self.wndOptions:FindChild("cnGivRedTxt"):SetText(n2);
	elseif wndHandler:GetName() == "cnGiversGreen" then
		self.tUserSettings.cnGivers["green"] = n
		self.wndOptions:FindChild("cnGivGreenTxt"):SetText(n2);
	elseif wndHandler:GetName() == "cnGiversBlue" then
		self.tUserSettings.cnGivers["blue"] = n
		self.wndOptions:FindChild("cnGivBlueTxt"):SetText(n2);
	elseif wndHandler:GetName() == "cnGiversAlpha" then
		self.tUserSettings.cnGivers["alpha"] = n
		self.wndOptions:FindChild("cnGivAlphaTxt"):SetText(n2);
	elseif wndHandler:GetName() == "cnPathRed" then
		self.tUserSettings.cnPath["red"] = n
		self.wndOptions:FindChild("cnPathRedTxt"):SetText(n2);
	elseif wndHandler:GetName() == "cnPathGreen" then
		self.tUserSettings.cnPath["green"] = n
		self.wndOptions:FindChild("cnPathGreenTxt"):SetText(n2);
	elseif wndHandler:GetName() == "cnPathBlue" then
		self.tUserSettings.cnPath["blue"] = n
		self.wndOptions:FindChild("cnPathBlueTxt"):SetText(n2);
	elseif wndHandler:GetName() == "cnPathAlpha" then
		self.tUserSettings.cnPath["alpha"] = n
		self.wndOptions:FindChild("cnPathAlphaTxt"):SetText(n2);
		
	elseif wndHandler:GetName() == "cnHarvestRed" then
		self.tUserSettings.cnHarvest["red"] = n
		self.wndOptions:FindChild("cnHarvestRedTxt"):SetText(n2);
	elseif wndHandler:GetName() == "cnHarvestGreen" then
		self.tUserSettings.cnHarvest["green"] = n
		self.wndOptions:FindChild("cnHarvestGreenTxt"):SetText(n2);
	elseif wndHandler:GetName() == "cnHarvestBlue" then
		self.tUserSettings.cnHarvest["blue"] = n
		self.wndOptions:FindChild("cnHarvestBlueTxt"):SetText(n2);
	elseif wndHandler:GetName() == "cnHarvestAlpha" then
		self.tUserSettings.cnHarvest["alpha"] = n
		self.wndOptions:FindChild("cnHarvestAlphaTxt"):SetText(n2);
	elseif wndHandler:GetName() == "cnFarmRed" then
		self.tUserSettings.cnFarm["red"] = n
		self.wndOptions:FindChild("cnFarmRedTxt"):SetText(n2);
	elseif wndHandler:GetName() == "cnFarmGreen" then
		self.tUserSettings.cnFarm["green"] = n
		self.wndOptions:FindChild("cnFarmGreenTxt"):SetText(n2);
	elseif wndHandler:GetName() == "cnFarmBlue" then
		self.tUserSettings.cnFarm["blue"] = n
		self.wndOptions:FindChild("cnFarmBlueTxt"):SetText(n2);
	elseif wndHandler:GetName() == "cnFarmAlpha" then
		self.tUserSettings.cnFarm["alpha"] = n
		self.wndOptions:FindChild("cnFarmAlphaTxt"):SetText(n2);
	end
	
	
	self:UpdateLineColors()
	self.wndOptions:FindChild("ccObjectivesShow"):SetBGColor(self.tColor["objective"])
	self.wndOptions:FindChild("ccGiversShow"):SetBGColor(self.tColor["giver"])
	self.wndOptions:FindChild("ccPathShow"):SetBGColor(self.tColor["path"])
	self.wndOptions:FindChild("ccHarvestShow"):SetBGColor(self.tColor["harvest"])
	self.wndOptions:FindChild("ccFarmShow"):SetBGColor(self.tColor["farm"])
end


-----------------------------------------------------------------------------------------------
-- GuideLine Instance
-----------------------------------------------------------------------------------------------
local GuideLineInst = GuideLine:new()
GuideLineInst:Init()
